from .fileutil import (isdir, isfile, joinpath, basename, cp, rm,
                       cptree, rmtree, move, mkdirs, copyfiles,
                       GlobMode, glob)
from .multiprocessutil import tqdm_imap


__all__ = [
    'isdir', 'isfile', 'joinpath', 'basename', 'cp', 'rm',
    'cptree', 'rmtree', 'move', 'mkdirs', 'copyfiles',
    'GlobMode', 'glob',

    'tqdm_imap'
]



import multiprocessing
from tqdm import tqdm


def tqdm_imap(func, args, n_processes=None):
    p = multiprocessing.Pool(n_processes)

    result = []
    for res in tqdm(p.imap(func, args), total=len(args)):
        result.append(res)

    p.close()
    p.join()

    return result


# Multiprocessing cannot be tested in pytest.
# We can test it here instead.
if __name__ == '__main__':
    def func(first, second, third):
        return first + second + third

    first = [str(i) for i in range(100)]
    second = '_m'
    third = '.png'

    # single processer version
    result_sp = [func(i, second, third) for i in first]

    # multi-processers version 1
    def func_wrapper(xyz):
        return func(*xyz)
    in_para_wrapper = [(i, second, third) for i in first]
    result_mp = tqdm_imap(func_wrapper, in_para_wrapper)
    assert result_sp == result_mp

    # multi-processers version 2
    from functools import partial
    partial_func = partial(func, second=second, third=third)
    result_mp = tqdm_imap(partial_func, first)
    assert result_sp == result_mp



from enum import Enum, unique, auto
from glob import glob as std_glob
import os
import shutil
from natsort import natsorted


isdir = os.path.isdir
isfile = os.path.isfile

joinpath = os.path.join
basename = os.path.basename

cp = shutil.copy
rm = os.remove
cptree = shutil.copytree
rmtree = shutil.rmtree
move = shutil.move


def mkdirs(path, mode=0o777):
    path = os.path.expanduser(path)
    os.makedirs(path, mode, exist_ok=True)


def copyfiles(src_paths, dst_dir, src_root=None):
    assert isdir(dst_dir)
    assert isinstance(src_paths, (tuple, list))

    for src_path in src_paths:
        cp(joinpath(src_root, src_path), dst_dir)


@unique
class GlobMode(Enum):
    FILE = auto()
    DIR = auto()
    ALL = auto()


def glob(root, pattern, mode=GlobMode.FILE, recursive=False):
    root = os.path.expanduser(root)
    pattern = joinpath('**', pattern) if recursive else pattern
    pathname = joinpath(root, pattern)

    result = std_glob(pathname, recursive=recursive)

    if mode == GlobMode.FILE:
        result = filter(isfile, result)
    elif mode == GlobMode.DIR:
        result = filter(isdir, result)
    else:  # GlobMode.ALL
        pass

    return natsorted(result)
